//----------------------------------------------------------------------------
//
//RMIChatClient.java, a text chat server program of a server/client pair,
//using RMI callback function.
//
//----------------------------------------------------------------------------

import java.rmi.*;
import java.rmi.server.*;
import java.util.Hashtable;
import java.util.Vector;
import java.util.Enumeration;
import java.io.*;


//RMI chat server application

public class RMIChatServer extends UnicastRemoteObject
        implements RMIChatIF
{
    private Hashtable userTable;    //a table of callback object for each client
    private String objName;

    //constructor

    public RMIChatServer(String s) throws java.rmi.RemoteException {
        super();
        objName = s;
        userTable = new Hashtable();
    }

    //-----------------------------------------------------------------------------------------------------------------
    //The next three functions implement the RMIChatIF interface

    //register user ID and callback object in server

    public boolean login(String userID,int rn, RMIChatNotify obj) throws java.rmi.RemoteException
    {
		userID=Integer.toString(rn)+userID;
        if (userTable.containsKey(userID)) {
            return false;
        }
        else {
            userTable.put(userID, obj);
            sendUserList();
            return true;
        }
    }

    //a client send chat message  to server

    public void sendMsg(String userID, String s , int room ) throws java.rmi.RemoteException , java.io.IOException
    {
        Enumeration enum = userTable.elements();
        while (enum.hasMoreElements()) {
            RMIChatNotify user = (RMIChatNotify)enum.nextElement();
            user.receiveMsg(userID, s , room );
        }


String blue = "/servers/cgi-http-class/htdoc/cps606fall99-cgi/cb99mba/blue.txt";
String green = "/servers/cgi-http-class/htdoc/cps606fall99-cgi/cb99mba/green.txt";
String red = "/servers/cgi-http-class/htdoc/cps606fall99-cgi/cb99mba/red.txt";
String yellow = "/servers/cgi-http-class/htdoc/cps606fall99-cgi/cb99mba/yellow.txt";

   switch(room)
   {
	   case 0 :
	   {
        PrintWriter outfile =  new PrintWriter( new FileWriter( blue , true ) );
        outfile.println(userID+":"+s);
        outfile.close();
        break;
       }

       case 1 :
	   	   {
	           PrintWriter outfile =  new PrintWriter( new FileWriter( green , true ) );
	           outfile.println(userID+":"+s);
	           outfile.close();
	           break;
	       }

	   case 2 :
	   {
        PrintWriter outfile =  new PrintWriter( new FileWriter( red , true ) );
        outfile.println(userID+":"+s);
        outfile.close();
        break;
       }

       case 3 :
	   {
	    PrintWriter outfile =  new PrintWriter( new FileWriter( yellow , true ) );
	    outfile.println(userID+":"+s);
	    outfile.close();
	    break;
	   }

} //end of switch


    }

    //unregister user ID and callback object in server

    public boolean logout(String userID, int rn , RMIChatNotify obj) throws java.rmi.RemoteException

    {
		userID=Integer.toString(rn)+userID;
        if ( ! userTable.containsKey(userID) ) {
            return false;
        }
        else {
            userTable.remove(userID);
            sendUserList();
            return true;
        }
    }

    //broadcast user list to all client

    private void sendUserList()  throws java.rmi.RemoteException {
        Vector ul = new Vector();

        //put the user list into a vector
        Enumeration enum1 = userTable.keys();
        while (enum1.hasMoreElements()) {
            ul.addElement( (String)enum1.nextElement() );
         }

         //broadcast user list
        Enumeration enum2 = userTable.elements();
        while (enum2.hasMoreElements()) {
            RMIChatNotify user = (RMIChatNotify)enum2.nextElement();
            user.receiveUserList(ul);
        }
    }

    //start server and register server in RMI naming service

    public static void main(String[] args) {

        // Create and install the security manager
        System.setSecurityManager(new RMISecurityManager());

        try {
            //register the RMI server
            RMIChatServer csobj = new RMIChatServer("RMIChatServer");

            Naming.rebind("//carver.npac.syr.edu:2345/RMIChatServer", csobj);

            System.out.println("RMIChatServer created and bound in the registry to the name RMIChatServer");
        }
        catch (Exception ex) {
            System.err.println("RMIChatServer: Exception-> " + ex);
            ex.printStackTrace();
        }
  }

}

